﻿using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Navigation;
using Microsoft.Phone.BackgroundAudio;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using PlaylistFilePlaybackAgent;

namespace PlaylistFilePlayer
{
    public partial class MainPage : PhoneApplicationPage
    {
        static string[] uris = 
        { 
            "http://www.archive.org/download/DebussyPreludesBookI/01No.1-DanseusesDeDelphes.mp3",
            "http://www.archive.org/download/DebussyPreludesBookI/02No.2-Voiles.mp3",
            "http://www.archive.org/download/DebussyPreludesBookI/03No.3-LeVentDansLaPlaine.mp3",
            "http://www.archive.org/download/DebussyPreludesBookI/04No.4-LesSonsEtLesParfumsTournentDansLairDuSoir.mp3",
            "http://www.archive.org/download/DebussyPreludesBookI/05No.5-LesCollinesDanacapri.mp3",
            "http://www.archive.org/download/DebussyPreludesBookI/06No.6-DesPasSurLaNeige.mp3",
            "http://www.archive.org/download/DebussyPreludesBookI/07No.7-CQuaVuLeVentDouest.mp3",
            "http://www.archive.org/download/DebussyPreludesBookI/08No.8-LaFilleAuxCheveuxDeLinNo.9-LaSrnadeInterrompue.mp3",
            "http://www.archive.org/download/DebussyPreludesBookI/09No.10-LaCathdraleEngloutie.mp3",
            "http://www.archive.org/download/DebussyPreludesBookI/10No.11-LaDanseDePuckNo.12-Minstrels.mp3"
        };
        static string[] titles = 
        { 
            "1. Danseuses de Delphes", 
            "2. Voiles", 
            "3. Le vent dans la plaine",
            "4. Les sons et les parfums tournent dans l'air du soir",
            "5. Les collines d'Anacapri",
            "6. Des pas sur la neige",
            "7. Cé qu'a vu le vent d'ouest",
            "8. La fille aux cheveux de lin\r\n9. La sérénade interrompue",
            "10. La cathédrale engloutie ",
            "11. La danse de Puck\r\n12. Minstrels "
        };

        Playlist playlist;
        bool settingSliderFromPlay;
        
        public MainPage()
        {
            InitializeComponent();

            // Create playlist
            playlist = new Playlist();

            // Build the playlist
            for (int i = 0; i < uris.Length; i++)
            {
                EnabledPlayerControls playerControls =
                        EnabledPlayerControls.Pause |
                        (i != 0 ? EnabledPlayerControls.SkipPrevious : 0) |
                        (i != uris.Length - 1 ? EnabledPlayerControls.SkipNext : 0);

                PlaylistTrack track = new PlaylistTrack
                    {
                        Source = uris[i],
                        Title = titles[i],
                        Artist = "Alfred Cortot",
                        Album = "Debussy: Preludes Book 1",
                        PlayerControls = playerControls
                    };
                playlist.Tracks.Add(track);
            }

            // Save it to isolated storage
            playlist.Save("playlist.xml");

            // Set fields of appbar buttons; otherwise they're null
            prevAppBarButton = this.ApplicationBar.Buttons[0] as ApplicationBarIconButton;
            playAppBarButton = this.ApplicationBar.Buttons[1] as ApplicationBarIconButton;
            pauseAppBarButton = this.ApplicationBar.Buttons[2] as ApplicationBarIconButton;
            nextAppBarButton = this.ApplicationBar.Buttons[3] as ApplicationBarIconButton;

            // Set PlayStateChanged handler
            BackgroundAudioPlayer.Instance.PlayStateChanged += OnBackgroundAudioPlayerPlayStateChanged;

            // Set CompositionTarget.Rendering handler
            CompositionTarget.Rendering += OnCompositionTargetRendering;
        }

        protected override void OnNavigatedTo(NavigationEventArgs args)
        {
            UpdateScreen();
            base.OnNavigatedTo(args);
        }

        void OnBackgroundAudioPlayerPlayStateChanged(object sender, EventArgs args)
        {
            UpdateScreen();
        }

        void UpdateScreen()
        {
            PlayState state = PlayState.Unknown;
            AudioTrack audioTrack = null;

            try
            {
                // Sometimes these property accesses will raise exceptions
                state = BackgroundAudioPlayer.Instance.PlayerState;
                audioTrack = BackgroundAudioPlayer.Instance.Track;
            }
            catch
            {
            }

            playerState.Text = state != PlayState.Unknown ? state.ToString() : null;

            if (audioTrack != null)
            {
                albumText.Text = audioTrack.Album;
                artistText.Text = audioTrack.Artist;
                trackText.Text = audioTrack.Title;

                positionSlider.Visibility = Visibility.Visible;

                prevAppBarButton.IsEnabled = 0 != (audioTrack.PlayerControls & EnabledPlayerControls.SkipPrevious);
                nextAppBarButton.IsEnabled = 0 != (audioTrack.PlayerControls & EnabledPlayerControls.SkipNext);
                playAppBarButton.IsEnabled = BackgroundAudioPlayer.Instance.PlayerState == PlayState.Paused;
                pauseAppBarButton.IsEnabled = BackgroundAudioPlayer.Instance.PlayerState == PlayState.Playing;
            }
            else
            {
                albumText.Text = null;
                artistText.Text = null;
                trackText.Text = null;

                positionSlider.Visibility = Visibility.Collapsed;

                prevAppBarButton.IsEnabled = false;
                playAppBarButton.IsEnabled = true;
                pauseAppBarButton.IsEnabled = false;
                nextAppBarButton.IsEnabled = false;
            }
        }

        void OnCompositionTargetRendering(object sender, EventArgs args)
        {
            AudioTrack audioTrack = null;
            TimeSpan position = TimeSpan.Zero;
            TimeSpan duration = TimeSpan.Zero;

            try
            {
                // Sometimes these property accesses will raise exceptions
                audioTrack = BackgroundAudioPlayer.Instance.Track;

                if (audioTrack != null)
                {
                    duration = audioTrack.Duration;
                    position = BackgroundAudioPlayer.Instance.Position;
                }
            }
            catch
            {
            }

            if (audioTrack != null)
            {
                elapsedDuration.Text = position.ToString("m\\:ss\\.f");
                totalDuration.Text = duration.ToString("m\\:ss\\.f");

                // Change the Slider position, but set a flag to 
                //  avoid the ValueChanged handler from being confused.
                settingSliderFromPlay = true;

                if (duration.Ticks > 0)
                    positionSlider.Value = (double)position.Ticks / duration.Ticks;
                else
                    positionSlider.Value = 0;

                settingSliderFromPlay = false;
            }
            else
            {
                elapsedDuration.Text = null;
                totalDuration.Text = null;
                positionSlider.Value = 0;
            }
        }

        // Application Bar Buttons:
        //  After one is clicked, disable it for further clicks
        //      to prevent multiple clicks while DLL or song is loading
        void OnPrevAppBarButtonClick(object sender, EventArgs args)
        {
            BackgroundAudioPlayer.Instance.SkipPrevious();
            prevAppBarButton.IsEnabled = false;
        }

        void OnPlayAppBarButtonClick(object sender, EventArgs args)
        {
            BackgroundAudioPlayer.Instance.Play();
            playAppBarButton.IsEnabled = false;
        }

        void OnPauseAppBarButtonClick(object sender, EventArgs args)
        {
            BackgroundAudioPlayer.Instance.Pause();
            pauseAppBarButton.IsEnabled = false;
        }

        void OnNextAppBarButtonClick(object sender, EventArgs args)
        {
            BackgroundAudioPlayer.Instance.SkipNext();
            nextAppBarButton.IsEnabled = false;
        }

        void OnSliderValueChanged(object sender, RoutedPropertyChangedEventArgs<double> args)
        {
            if (!settingSliderFromPlay)
            {
                AudioTrack audioTrack = null;
                TimeSpan duration = TimeSpan.Zero;

                try
                {
                    // Sometimes these property accesses will raise exceptions
                    audioTrack = BackgroundAudioPlayer.Instance.Track;

                    if (audioTrack != null)
                        duration = audioTrack.Duration;
                }
                catch
                {
                }

                if (audioTrack == null)
                    return;

                long ticks = (long)(args.NewValue * duration.Ticks);
                BackgroundAudioPlayer.Instance.Position = new TimeSpan(ticks);
            }
        }
    }
}