﻿using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Navigation;
using Microsoft.Phone.BackgroundAudio;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;

namespace PlaylistPlayer
{
    public partial class MainPage : PhoneApplicationPage
    {
        bool settingSliderFromPlay;

        public MainPage()
        {
            InitializeComponent();

            // Set fields of appbar buttons; otherwise they're null
            prevAppBarButton = this.ApplicationBar.Buttons[0] as ApplicationBarIconButton;
            playAppBarButton = this.ApplicationBar.Buttons[1] as ApplicationBarIconButton;
            pauseAppBarButton = this.ApplicationBar.Buttons[2] as ApplicationBarIconButton;
            nextAppBarButton = this.ApplicationBar.Buttons[3] as ApplicationBarIconButton;

            // Set PlayStateChanged handler
            BackgroundAudioPlayer.Instance.PlayStateChanged += OnBackgroundAudioPlayerPlayStateChanged;

            // Set CompositionTarget.Rendering handler
            CompositionTarget.Rendering += OnCompositionTargetRendering;
        }

        protected override void OnNavigatedTo(NavigationEventArgs args)
        {
            UpdateScreen();
            base.OnNavigatedTo(args);
        }

        void OnBackgroundAudioPlayerPlayStateChanged(object sender, EventArgs args)
        {
            UpdateScreen();
        }

        void UpdateScreen()
        {
            PlayState state = PlayState.Unknown;
            AudioTrack audioTrack = null;

            try
            {
                // Sometimes these property accesses will raise exceptions
                state = BackgroundAudioPlayer.Instance.PlayerState;
                audioTrack = BackgroundAudioPlayer.Instance.Track;
            }
            catch
            {
            }

            playerState.Text = state != PlayState.Unknown ? state.ToString() : null;

            if (audioTrack != null)
            {
                albumText.Text = audioTrack.Album;
                artistText.Text = audioTrack.Artist;
                trackText.Text = audioTrack.Title;

                positionSlider.Visibility = Visibility.Visible;

                prevAppBarButton.IsEnabled = 0 != (audioTrack.PlayerControls & EnabledPlayerControls.SkipPrevious);
                nextAppBarButton.IsEnabled = 0 != (audioTrack.PlayerControls & EnabledPlayerControls.SkipNext);
                playAppBarButton.IsEnabled = BackgroundAudioPlayer.Instance.PlayerState == PlayState.Paused;
                pauseAppBarButton.IsEnabled = BackgroundAudioPlayer.Instance.PlayerState == PlayState.Playing;
            }
            else
            {
                albumText.Text = null;
                artistText.Text = null;
                trackText.Text = null;

                positionSlider.Visibility = Visibility.Collapsed;

                prevAppBarButton.IsEnabled = false;
                playAppBarButton.IsEnabled = true;
                pauseAppBarButton.IsEnabled = false;
                nextAppBarButton.IsEnabled = false;
            }
        }

        void OnCompositionTargetRendering(object sender, EventArgs args)
        {
            AudioTrack audioTrack = null;
            TimeSpan position = TimeSpan.Zero;
            TimeSpan duration = TimeSpan.Zero;

            try
            {
                // Sometimes these property accesses will raise exceptions
                audioTrack = BackgroundAudioPlayer.Instance.Track;

                if (audioTrack != null)
                {
                    duration = audioTrack.Duration;
                    position = BackgroundAudioPlayer.Instance.Position;
                }
            }
            catch
            {
            }

            if (audioTrack != null)
            {
                elapsedDuration.Text = position.ToString("m\\:ss\\.f");
                totalDuration.Text = duration.ToString("m\\:ss\\.f");

                // Change the Slider position, but set a flag to 
                //  avoid the ValueChanged handler from being confused.
                settingSliderFromPlay = true;

                if (duration.Ticks > 0)
                    positionSlider.Value = (double)position.Ticks / duration.Ticks;
                else
                    positionSlider.Value = 0;

                settingSliderFromPlay = false;
            }
            else
            {
                elapsedDuration.Text = null;
                totalDuration.Text = null;
                positionSlider.Value = 0;
            }
        }

        // Application Bar Buttons:
        //  After one is clicked, disable it for further clicks
        //      to prevent multiple clicks while DLL or song is loading
        void OnPrevAppBarButtonClick(object sender, EventArgs args)
        {
            BackgroundAudioPlayer.Instance.SkipPrevious();
            prevAppBarButton.IsEnabled = false;
        }

        void OnPlayAppBarButtonClick(object sender, EventArgs args)
        {
            BackgroundAudioPlayer.Instance.Play();
            playAppBarButton.IsEnabled = false;
        }

        void OnPauseAppBarButtonClick(object sender, EventArgs args)
        {
            BackgroundAudioPlayer.Instance.Pause();
            pauseAppBarButton.IsEnabled = false;
        }

        void OnNextAppBarButtonClick(object sender, EventArgs args)
        {
            BackgroundAudioPlayer.Instance.SkipNext();
            nextAppBarButton.IsEnabled = false;
        }

        void OnSliderValueChanged(object sender, RoutedPropertyChangedEventArgs<double> args)
        {
            if (!settingSliderFromPlay)
            {
                AudioTrack audioTrack = null;
                TimeSpan duration = TimeSpan.Zero;

                try
                {
                    // Sometimes these property accesses will raise exceptions
                    audioTrack = BackgroundAudioPlayer.Instance.Track;

                    if (audioTrack != null)
                        duration = audioTrack.Duration;
                }
                catch
                {
                }

                if (audioTrack == null)
                    return;

                long ticks = (long)(args.NewValue * duration.Ticks);
                BackgroundAudioPlayer.Instance.Position = new TimeSpan(ticks);
            }
        }
    }
}